/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package d800udemo;

import com.sun.jna.Library;
import com.sun.jna.Native;

/**
 *
 * @author Administrator
 */
public interface D800UInteface extends Library {

    D800UInteface MasterComInstance = (D800UInteface) Native.loadLibrary("MasterCom", D800UInteface.class);
    D800UInteface MasterRDInstance = (D800UInteface) Native.loadLibrary("MasterRD", D800UInteface.class);

    //******** 功能：初始化串口 *******************************
    //  参数：port：串口号，取值为1～8
    //        baud：为通讯波特率4800～115200,默认115200
    //  返回：成功则返回0
    //*************************************************************
    public int rf_init_com(int port, long baud);

    //******** 功能：指定设备标识 *****************************
    //  参数：icdev：通讯设备标识符，0-65536
    //  返回：成功返回0
    //**************************************************************
    int rf_init_device_number(short icdev);

    //******** 功能：读取设备标识 *****************************
    //  参数：Icdev：通讯设备标识符
    //  返回：成功返回0
    //**************************************************************
    int rf_get_device_number(short Icdev);

    //*******  功能：控制LED亮或者灭 ***************
    //  参数：icdev：通讯设备标识符
    //        color：LED颜色
    //  返回：成功则返回0
    //        color=0:所有LED都灭掉
    //        color=1:blue led 
	//        color=2:red  led
    //*********************************************************
    int rf_light(short icdev, byte color);
	
	 //*******  功能：控制蜂鸣器开关 ***************
    //  参数：icdev：通讯设备标识符
    //        msec：延时，响的时间长度，单位10ms
    //  返回：成功则返回0   
    //*********************************************************
    int rf_beep(short icdev, byte msec);

    //*******  功能：关闭或启动读写卡器天线发射 ***************
    //  参数：icdev：通讯设备标识符
    //        model：天线状态
    //  返回：成功则返回0
    //  说明：model=0:关闭天线
    //        model=1:开启天线
    //*********************************************************
    int rf_antenna_sta(short icdev, byte model);

    //*******  功能：设置读写卡器非接触工作方式为 *************/
    //              ISO14443 TYPE A OR ISO14443 TYPE B
    //  参数：icdev：通讯设备标识符
    //        type:  读写卡器工作方式
    //  返回：成功则返回0
    //  说明：type='A':设置为TYPE_A方式
    //        type='B':设置为TYPE_B方式
    //        type='r':设置为AT88RF020卡方式
    //*********************************************************
    int rf_init_type(short icdev, char type);

    //******** 功能：寻ISO14443-3 TYPE_A 卡 *******************
    //  参数：icdev：  通讯设备标识符
    //        model：  寻卡模式
    //        TagType：返回卡类型值
    //  返回：成功则返回0
    //  说明：mode=0x26:寻未进入休眠状态的卡
    //        mode=0x52:寻所有状态的卡
    //*********************************************************
    int rf_request(short icdev, byte model, String TagType);

    //********* 功能：ISO14443-3 TYPE_A 卡防冲撞 **************
    //  参数：icdev：  通讯设备标识符
    //        bcnt：   卡序列号字节数，取值4、7、10，Mifare 卡取值4
    //        pSnr：   返回的卡序列号
    //        pRLength:卡序列号长度
    //  返回：成功则返回0
    //*********************************************************
    int rf_anticoll(short icdev, byte bcnt, byte[] pSnr, String pRLength);
    /*例：int status
     unsigned char icdev;
     unsigned char snr[MAX_RF_BUFFER];
     unsigned char len;
     status = rf_anticoll(icdev,4,snr,&len);      
     */

    //******** 功能：锁定一张ISO14443-3 TYPE_A 卡 *************
    //  参数：icdev：通讯设备标识符
    //        pSnr： 卡序列号
    //        srcLen:卡序列号长度，MifareOne卡该值等于4
    //        Size： 返回卡容量
    //  返回：成功则返回0
    //*********************************************************
    int rf_select(short icdev, byte[] pSnr, byte srcLen, String Size);

    //*******  功能：读取Mifare One 卡一块数据 ****************
    //  参数：icdev： 通讯设备标识符
    //        block： M1卡绝对块号
    //        pData： 读出数据
    //        pLen:   读出数据的长度
    //  返回：成功则返回0
    //*********************************************************
    int rf_M1_read(short icdev, byte block, byte[] pData, String pLen);
    /*例：int status
     unsigned short icdev
     unsigned char pData[MAX_RF_BUFFER];
     unsigned char len;
     status = rf_M1_read(icdev,0,pData,&len);
     */

    //*******  功能：向Mifare One 卡中写入一块数据 ************/
    //  参数：icdev：通讯设备标识符
    //        block：M1卡绝对块号
    //        data： 写入的数据，16 字节
    //  返回：成功则返回0
    //*********************************************************/
    int rf_M1_write(short icdev, byte block, byte[] data);

    //***** 功能：用指定的密钥验证Mifare One 卡*****************
    //  参数：icdev：通讯设备标识符
    //        model：密码验证模式
    //        block：要验证密码的绝对块号
    //        key：  密钥内容，6 字节
    //  返回：成功则返回0
    //  说明：model=0x60:验证A密钥
    //        model=0x61:验证B密钥
    //**********************************************************
    int rf_M1_authentication2(short icdev, byte model, byte block, byte[] key);

    //***** 功能：命令已激活的TYPE_A卡进入HALT状态*****************
    //  参数：icdev：通讯设备标识符
    //返回：成功返回0
    //说明：卡片接收到此命令后退出激活状态
    //**********************************************************
    int rf_halt(short icdev);
}
