﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


using System.Runtime.InteropServices;



namespace Mifare1K
{

    public partial class Mifare_1K : Form
    {
        [DllImport("kernel32.dll")]
        static extern void Sleep(int dwMilliseconds);

        [DllImport("MasterRD.dll")]
        static extern int lib_ver(ref uint pVer);

        [DllImport("MasterRD.dll")]
        static extern int rf_init_com(int port, int baud);

        [DllImport("MasterRD.dll")]
        static extern int rf_ClosePort();

        [DllImport("MasterRD.dll")]
        static extern int rf_beep(short icdev, byte delay);

        [DllImport("MasterRD.dll")]
        static extern int rf_light(short icdev, byte Ledcolor);
       
        [DllImport("MasterRD.dll")]
        static extern int rf_antenna_sta(short icdev, byte mode);

        [DllImport("MasterRD.dll")]
        static extern int rf_init_type(short icdev, byte type);

        [DllImport("MasterRD.dll")]
        static extern int rf_request(short icdev, byte mode, ref ushort pTagType);

        
        [DllImport("MasterRD.dll")]
        static extern int rf_halt(short icdev);
        
        [DllImport("MasterRD.dll")]
        static extern int rf_M1_read(short icdev, byte adr, IntPtr pData, ref byte pLen);

        
        [DllImport("MasterRD.dll")]
        static extern int rf_ul_select(short icdev, IntPtr pSnr, ref byte pRLength);

        [DllImport("MasterRD.dll")]
        static extern int rf_ul_write(short icdev, byte page, IntPtr pData);


        //
        bool bConnectedDevice;/*是否连接上设备*/

        static char[] hexDigits = { 
            '0','1','2','3','4','5','6','7',
            '8','9','A','B','C','D','E','F'};

        public static byte GetHexBitsValue(byte ch)
        {
            byte sz = 0;
            if (ch <= '9' && ch >= '0')
                sz = (byte)(ch - 0x30);
            if (ch <= 'F' && ch >= 'A')
                sz = (byte)(ch - 0x37);
            if (ch <= 'f' && ch >= 'a')
                sz = (byte)(ch - 0x57);

            return sz;
        }
        //

        #region byteHEX
        /// <summary>
        /// 单个字节转字字符.
        /// </summary>
        /// <param name="ib">字节.</param>
        /// <returns>转换好的字符.</returns>
        public static String byteHEX(Byte ib)
        {
            String _str = String.Empty;
            try
            {
                char[] Digit = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A',
			    'B', 'C', 'D', 'E', 'F' };
                char[] ob = new char[2];
                ob[0] = Digit[(ib >> 4) & 0X0F];
                ob[1] = Digit[ib & 0X0F];
                _str = new String(ob);
            }
            catch (Exception)
            {
                new Exception("对不起有错。");
            }
            return _str;

        }
        #endregion

        public static string ToHexString(byte[] bytes)
        {
            String hexString = String.Empty;
            for (int i = 0; i < bytes.Length; i++)
                hexString += byteHEX(bytes[i]);

            return hexString;
        }



        public static byte[] ToDigitsBytes(string theHex)
        {
            byte[] bytes = new byte[theHex.Length / 2 + (((theHex.Length % 2) > 0) ? 1 : 0)];
            for (int i = 0; i < bytes.Length; i++)
            {
                char lowbits = theHex[i * 2];
                char highbits;

                if ((i * 2 + 1) < theHex.Length)
                    highbits = theHex[i * 2 + 1];
                else
                    highbits = '0';

                int a = (int)GetHexBitsValue((byte)lowbits);
                int b = (int)GetHexBitsValue((byte)highbits);
                bytes[i] = (byte)((a << 4) + b);
            }

            return bytes;
        }

        /**/
        public Mifare_1K()
        {
            InitializeComponent();
        }

        private void Mifare_1K_Load(object sender, EventArgs e)
        {
            bConnectedDevice = false;
                     
            comboPage.SelectedIndex = 7;
            combo_URI_Identifier.SelectedIndex = 1;

            tscbxPort.SelectedIndex = 1;   // default port COM2
            tscbxBaud.SelectedIndex = 6;  // default 115200
           

        }


       
        private void tsbtnConnect_Click(object sender, EventArgs e)
        {/*toolbar button[connect] clicked*/
            if (!bConnectedDevice)
            {
                int port = 0;
                int baud = 0;
                int status;

                port = Convert.ToInt32(tscbxPort.Text);
                baud = Convert.ToInt32(tscbxBaud.Text);

                status = rf_init_com(port, baud);
                if (0 == status)
                {
                    tsbtnConnect.Text = "Disconnect";
                    bConnectedDevice = true;
                    MessageBox.Show("Connect device success!");
                }
                else
                {
                    string strError;
                    strError = "Connect device failed";
                    bConnectedDevice = false;
                    MessageBox.Show(strError, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                rf_ClosePort();
                bConnectedDevice = false;
                tsbtnConnect.Text = "Connect";
            }

        }

               
        // This command include 2 commands: request and UL_Select,finally get the 7 bytes UID of Ultralight tag.
        private void btnUID7_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            // byte type = (byte)'A';//mifare one type is A 卡询卡方式为A
            byte mode = 0x26;  // Request the card which is not halted.
            ushort TagType = 0;
            //  byte bcnt = 0x04;//mifare 卡都用4, hold on 4
            IntPtr pSnr;
            byte len = 255;
            // sbyte size = 0;


            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            txtUID7.Text = "";

            pSnr = Marshal.AllocHGlobal(1024);

            for (int i = 0; i < 2; i++)
            {
                //  Sleep(50);     // After open the antenna, it needs about 50ms delay before request.
                status = rf_request(icdev, mode, ref TagType);//搜寻没有休眠的卡，request card  
                if (status != 0)
                    continue;

                status = rf_ul_select(icdev, pSnr, ref len);//防冲突得到返回卡的序列号,--get the card UID      
                if (status != 0)
                    continue;

                byte[] szBytes = new byte[len];

                for (int j = 0; j < len; j++)
                {
                    szBytes[j] = Marshal.ReadByte(pSnr, j);
                }

                String m_cardNo = String.Empty;

                for (int q = 0; q < len; q++)
                {
                    m_cardNo += byteHEX(szBytes[q]);
                }
                txtUID7.Text = m_cardNo;

                break;
            }
            int color = 2;// 2:red led light ,1: blue light,0: led off
            rf_light(icdev, (byte)color);

            Marshal.FreeHGlobal(pSnr);
        }

        // This function read out 4 pages of the Ultralight/NTAG203
        private void btnReadULpage_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            byte npage = 0x00;

            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            txtPageRead.Text = "";

            npage = Convert.ToByte(comboPage.Text);

            IntPtr dataBuffer = Marshal.AllocHGlobal(256);

            int j;
            byte cLen = 0;
            status = rf_M1_read(icdev, (byte)(npage), dataBuffer, ref cLen);

            if (status != 0 || cLen != 16)
            {
                MessageBox.Show("rf_M1_read failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Marshal.FreeHGlobal(dataBuffer);
                return;
            }

            byte[] bytesData = new byte[4];      // 4 bytes for one page
            for (j = 0; j < bytesData.Length; j++)
                bytesData[j] = Marshal.ReadByte(dataBuffer, j);

            txtPageRead.Text = ToHexString(bytesData);


            Marshal.FreeHGlobal(dataBuffer);
        }

        // This command will write the 4 bytes data into the page.
        // Do NOT write page 2,page3 before you understand why and how,more detail, please see the NTAG213 or else chip datasheet
        private void btnWriteULpage_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            byte adr;
            int i;

            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            adr = (byte)(Convert.ToByte(comboPage.Text));
            byte[] bytesBlock;
            bytesBlock = ToDigitsBytes(txtPageWrite.Text);

            IntPtr dataBuffer = Marshal.AllocHGlobal(1024);
            for (i = 0; i < bytesBlock.Length; i++)
                Marshal.WriteByte(dataBuffer, i, bytesBlock[i]);
            status = rf_ul_write(icdev, adr, dataBuffer);
            Marshal.FreeHGlobal(dataBuffer);

            if (status != 0)
            {
                MessageBox.Show("rf_ul_write failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        // This command will read out the text string by NDEF formatted. 
        // More detail please reference the NTAG213 datasheet or else NDEF files.
        // Read the UID of the NTAG CHIP befor this command!
        private void btn_NFC_Text_Read_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            byte npage = 0x00;
            byte txtCode = 0x54;
            int nPageCount = 0;

            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            text_NFC_Text_Input.Text = "";

            npage = Convert.ToByte(6);// The special code and length data saved in page 6

            IntPtr dataBuffer = Marshal.AllocHGlobal(256);

            int j;
            byte cLen = 0;
            status = rf_M1_read(icdev, (byte)(npage), dataBuffer, ref cLen); // This command will read out 4 pages=4*4=16(bytes).

            if (status != 0 || cLen != 16)
            {
                MessageBox.Show("rf_M1_read failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Marshal.FreeHGlobal(dataBuffer);
                return;
            }

            byte[] bytesData = new byte[4];      // 4 bytes for one page
            for (j = 0; j < bytesData.Length; j++)
                bytesData[j] = Marshal.ReadByte(dataBuffer, j);
            if (bytesData[2] != txtCode)
            {
                MessageBox.Show("Ilegal NFC Text!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Marshal.FreeHGlobal(dataBuffer);
                return;
            }
           
            byte nLen = 0;
            nLen = bytesData[1]; // Length of the text file including 2 preamble bytes and 1 postamble byte(0xfe)
           
            nPageCount = nLen / 4;
            j = nLen % 4;
            if (j > 0)
                nPageCount = nPageCount + 1;


            byte[] bytesReceiveData = new byte[256];
            int i;
            for (j = 7; j < (7 + nPageCount); j++)  // read from page 7
            {
                status = rf_M1_read(icdev, (byte)(j), dataBuffer, ref cLen);
                if (status != 0 || cLen != 16)
                {
                    MessageBox.Show("Read page failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Marshal.FreeHGlobal(dataBuffer);
                    return;
                }

                for (i = 0; i < 4; i++)
                    bytesReceiveData[4 * (j - 7) + i] = Marshal.ReadByte(dataBuffer, i);
            }

            byte[] bytesAsciiData = new byte[nLen - 3];
            for (i = 2; i < nLen - 1; i++)
                bytesAsciiData[i - 2] = Marshal.ReadByte(bytesReceiveData, i); // get the true text ascii code
            text_NFC_Text_Input.Text = System.Text.Encoding.ASCII.GetString(bytesAsciiData).Trim(); // Show the text string saved in the nfc chip.
           
            Marshal.FreeHGlobal(dataBuffer);
            int delaytime = 50;
            rf_beep(icdev, (byte)delaytime);
        }

        // This command will convert the input text string to Ascii hex array then write into the NTAG chip page by page.
        private void btn_NFC_Text_Write_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            byte npage;
            int i;

            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            int Len;
            Len = text_NFC_Text_Output.Text.Trim().Length;
            Len = Len + 3;  // add Encode bytes(2) and tailer code byte(1)
            
            IntPtr dataBuffer = Marshal.AllocHGlobal(1024);
            npage = (byte)(Convert.ToByte(4));
            byte[] page4_buf = new byte[] { 0x01, 0x03, 0xa0, 0x10 }; ////Page4_buf[4]={0x01,0x03,0xa0,0x10};//0c
            for (i = 0; i < page4_buf.Length; i++)
                Marshal.WriteByte(dataBuffer, i, page4_buf[i]);
            status = rf_ul_write(icdev, npage, dataBuffer);
           
            if (status != 0)
            {
                MessageBox.Show("Write page 4 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            npage = (byte)(Convert.ToByte(5));
            byte[] page5_buf = new byte[] { 0x44, 0x03, (byte)(Len + 4), 0xd1 };
            for (i = 0; i < page5_buf.Length; i++)
                Marshal.WriteByte(dataBuffer, i, page5_buf[i]);
            status = rf_ul_write(icdev, npage, dataBuffer);
            
            if (status != 0)
            {
                MessageBox.Show("Write page 5 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            npage = (byte)(Convert.ToByte(6));
            byte[] page6_buf = new byte[] { 0x01, (byte)Len, 0x54, 0x02 };
            for (i = 0; i < page6_buf.Length; i++)
                Marshal.WriteByte(dataBuffer, i, page6_buf[i]);
            status = rf_ul_write(icdev, npage, dataBuffer);
            
            if (status != 0)
            {
                MessageBox.Show("Write page 6 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            int tPage = 0;   // Total Pages
            tPage = Len / 4;
            int j = Len % 4;
            int k= 0;  // fill zero counter
            if (j > 0)
            { 
                tPage = tPage + 1;
                k = 4 - j;
            }
            string sZero="00";
            string sFillTail=string.Empty;
            if (k > 0)
            {
                for(i=0;i<k;i++)
                    sFillTail += sZero;
            }

            byte[] bEncode = new byte[] { 0x65, 0x6e };
            string sEncode = string.Empty;
            for (i = 0; i < bEncode.Length; i++)
            {
                sEncode += Convert.ToString(bEncode[i], 16);
            }

            byte[] bTrailer = new byte[] { 0xfe };
            string sTailer = string.Empty;
            for (i = 0; i < bTrailer.Length; i++)
            {
                sTailer += Convert.ToString(bTrailer[i], 16);
            }

            byte[] bytesBlock;
            bytesBlock = Encoding.ASCII.GetBytes(text_NFC_Text_Output.Text.Trim());// Convert the string input to ascii code bytes.
            string sInput = string.Empty;
            for (i = 0; i < bytesBlock.Length; i++)
            {
                sInput += Convert.ToString(bytesBlock[i], 16);
            }

            string sComb = string.Empty;
            sComb += sEncode;
            sComb += sInput;
            sComb += sTailer;
            sComb += sFillTail;
            bytesBlock = ToDigitsBytes(sComb);    // convert to hex byte string  
           
            int ofs;
            for (j = 7; j < tPage + 7; j++)
            {                
                ofs=(j - 7) *4;                
                for (i = 0; i < 4; i++)
                { Marshal.WriteByte(dataBuffer, i, bytesBlock[i +ofs]); } // copy to pointer buffer               
                status = rf_ul_write(icdev, (byte)j, dataBuffer);               
                if (status != 0)
                {
                    MessageBox.Show("rf_ul_write failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            Marshal.FreeHGlobal(dataBuffer);
            int color = 2;// 2:red led light ,1: blue light,0: led off
            rf_light(icdev, (byte)color);
            int delaytime = 50;
            rf_beep(icdev, (byte)delaytime);
            color = 0;
            rf_light(icdev, (byte)color);
            color = 1;
            rf_light(icdev, (byte)color);
            MessageBox.Show("Write NFC text finish!", "OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
           
        }

        // This command will read out the URI string by NDEF formatted. 
        // More detail please reference the NTAG213 datasheet or else NDEF files.
        // Read the UID of the NTAG CHIP befor this command!
        private void btn_NFC_URI_Read_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            byte npage = 0x00;
            byte URICode = 0x55;
            int nPageCount = 0;

            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            text_URI_Read.Text = "";

            npage = Convert.ToByte(6);// The special code and length data saved in page 6

            IntPtr dataBuffer = Marshal.AllocHGlobal(256);

            int j;
            byte cLen = 0;
            status = rf_M1_read(icdev, (byte)(npage), dataBuffer, ref cLen); // This command will read out 4 pages=4*4=16(bytes).

            if (status != 0 || cLen != 16)
            {
                MessageBox.Show("Read page 6 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Marshal.FreeHGlobal(dataBuffer);
                return;
            }

            byte[] bytesData = new byte[4];      // 4 bytes for one page
            for (j = 0; j < bytesData.Length; j++)
                bytesData[j] = Marshal.ReadByte(dataBuffer, j);
            if (bytesData[2] != URICode)
            {
                MessageBox.Show("Ilegal NFC URI!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Marshal.FreeHGlobal(dataBuffer);
                return;
            }
            
            byte iDentifier;
            iDentifier = bytesData[3];
            byte nLen = 0;
            nLen = bytesData[1]; // Length of the URI 
            
            nPageCount = nLen / 4;
            j = nLen % 4;
            if (j > 0)
                nPageCount = nPageCount + 1;


            byte[] bytesReceiveData = new byte[256];
            int i;
            for (j = 7; j < (7 + nPageCount); j++)  // read from page 7
            {
                status = rf_M1_read(icdev, (byte)(j), dataBuffer, ref cLen);
                if (status != 0 || cLen != 16)
                {
                    MessageBox.Show("Read page failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Marshal.FreeHGlobal(dataBuffer);
                    return;
                }

                for (i = 0; i < 4; i++)
                    bytesReceiveData[4 * (j - 7) + i] = Marshal.ReadByte(dataBuffer, i);
            }

            byte[] bytesAsciiData = new byte[nLen];
            for (i = 0; i < nLen - 1; i++)
                bytesAsciiData[i] = Marshal.ReadByte(bytesReceiveData, i); // get the true text ascii code

            string sIdentifier=string.Empty;
            switch(iDentifier)
            {
                case 0x00:
                    sIdentifier = "";
                    break;
                case 0x01:
                    sIdentifier = "http://www.";
                    break;
                case 0x02:
                    sIdentifier = "https://www.";
                    break;
                case 0x03:
                    sIdentifier = "http://";
                    break;
                case 0x04:
                    sIdentifier = "https://";
                    break;
                default:
                    break;

            }

            text_URI_Read.Text =sIdentifier+ System.Text.Encoding.ASCII.GetString(bytesAsciiData).Trim(); // Show the text string saved in the nfc chip.
            
            Marshal.FreeHGlobal(dataBuffer);
            int delaytime = 50;
            rf_beep(icdev, (byte)delaytime);
        }

        // This command will convert the input URI string to Ascii hex array then write into the NTAG chip page by page.
        private void btn_NFC_URI_Write_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            byte npage;
            int i;
            byte bIdentifier;

            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            bIdentifier = (byte)combo_URI_Identifier.SelectedIndex;

            int Len;
            Len = text_URI_Write.Text.Trim().Length;//  
           
            
            IntPtr dataBuffer = Marshal.AllocHGlobal(1024);
            npage = (byte)(Convert.ToByte(4));
            byte[] page4_buf = new byte[] { 0x01, 0x03, 0xa0, 0x10 }; 
            for (i = 0; i < page4_buf.Length; i++)
                Marshal.WriteByte(dataBuffer, i, page4_buf[i]);
            status = rf_ul_write(icdev, npage, dataBuffer);

            if (status != 0)
            {
                MessageBox.Show("Write page 4 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            npage = (byte)(Convert.ToByte(5));
            byte[] page5_buf = new byte[] { 0x44, 0x03, (byte)(Len+5), 0xd1 };
            for (i = 0; i < page5_buf.Length; i++)
                Marshal.WriteByte(dataBuffer, i, page5_buf[i]);
            status = rf_ul_write(icdev, npage, dataBuffer);

            if (status != 0)
            {
                MessageBox.Show("Write page 5 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            npage = (byte)(Convert.ToByte(6));
            byte[] page6_buf = new byte[] { 0x01, (byte)(Len + 1), 0x55, bIdentifier };
            for (i = 0; i < page6_buf.Length; i++)
                Marshal.WriteByte(dataBuffer, i, page6_buf[i]);
            status = rf_ul_write(icdev, npage, dataBuffer);

            if (status != 0)
            {
                MessageBox.Show("Write page 6 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            int tPage = 0;   // Total Pages
            tPage = Len / 4;
            int j = Len % 4;
            int k = 0;  // fill zero counter
            if (j > 0)
            {
                tPage = tPage + 1;
                k = 4 - j;
            }
            string sZero = "00";
            string sFillTail = string.Empty;
            if (k > 0)
            {
                for (i = 0; i < k; i++)
                    sFillTail += sZero;
            }
            
            byte[] bTrailer = new byte[] { 0xfe };
            string sTailer = string.Empty;
            for (i = 0; i < bTrailer.Length; i++)
            {
                sTailer += Convert.ToString(bTrailer[i], 16);
            }

            byte[] bytesBlock;
            bytesBlock = Encoding.ASCII.GetBytes(text_URI_Write.Text.Trim());// Convert the string input to ascii code bytes.
            string sInput = string.Empty;
            for (i = 0; i < bytesBlock.Length; i++)
            {
                sInput += Convert.ToString(bytesBlock[i], 16);
            }

            string sComb = string.Empty;

            sComb += sInput;
            sComb += sTailer;
            sComb += sFillTail;
            bytesBlock = ToDigitsBytes(sComb);    // convert to hex byte string  

            
            int ofs;
            for (j = 7; j < tPage + 7; j++)
            {
                ofs = (j - 7) * 4;
                for (i = 0; i < 4; i++)
                { Marshal.WriteByte(dataBuffer, i, bytesBlock[i + ofs]); } // copy to pointer buffer               
                status = rf_ul_write(icdev, (byte)j, dataBuffer);
                if (status != 0)
                {
                    MessageBox.Show("rf_ul_write failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            Marshal.FreeHGlobal(dataBuffer);
            int color = 2;// 2:red led light ,1: blue light,0: led off
            rf_light(icdev, (byte)color);
            int delaytime = 50;
            rf_beep(icdev, (byte)delaytime);
            color = 0;
            rf_light(icdev, (byte)color);
            color = 1;
            rf_light(icdev, (byte)color);
            MessageBox.Show("Write NFC URI finish!", "OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // This command will LOCK the chip forever and it is read only.
        private void btn_LOCK_NFC_Click(object sender, EventArgs e)
        {
            short icdev = 0x0000;
            int status;
            byte npage;
            int i;
            
            if (!bConnectedDevice)
            {
                MessageBox.Show("Not connect to device!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
           

            IntPtr dataBuffer = Marshal.AllocHGlobal(1024);
            npage = (byte)(Convert.ToByte(2));
            byte[] page2_buf = new byte[] { 0xff, 0xff, 0xff, 0xff };
            for (i = 0; i < page2_buf.Length; i++)
                Marshal.WriteByte(dataBuffer, i, page2_buf[i]);
            status = rf_ul_write(icdev, npage, dataBuffer);
            Marshal.FreeHGlobal(dataBuffer);
            if (status != 0)
            {
                MessageBox.Show("Write page 2 failed!!", "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            int color = 2;// 2:red led light ,1: blue light,0: led off
            rf_light(icdev, (byte)color);
            int delaytime = 50;
            rf_beep(icdev, (byte)delaytime);
            color = 0;
            rf_light(icdev, (byte)color);
            color = 1;
            rf_light(icdev, (byte)color);
            MessageBox.Show("LOCK NFC finish!", "OK", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // This command will open the web link read out on the text box.
        private void btn_OpenWebLink_Click(object sender, EventArgs e)
        {
            //调用IE浏览器   use iexplore    
            System.Diagnostics.Process.Start("iexplore.exe", text_URI_Read.Text.Trim());
            
          //调用系统默认的浏览器 use the default explore  
            System.Diagnostics.Process.Start(text_URI_Read.Text.Trim());
        }
    }
}